#!/bin/bash
#
# Script to monitor replication and safely purge binlogs. Exits with status
# 0 if replication is working, else exits with status 1. Optionally set up
# to send alerts vi SMTP or SNMP or to display STDOUT useful for an SNMP GET
# call.
#
# Periodically run this on the master database node from crontab.
#
# Note: You *MUST* configure SLAVE below and, if set, NOTIFY details
#
# Note: You *MUST* configure REPLICATION CLIENT privilege on the slave (which
#       may have already be done during the add_slave_user.sh step of setting
#       up a replicated database):
#
# GRANT REPLICATION CLIENT ON *.* TO 'DBUSER'@'MASTER' IDENTIFIED BY 'DBPWD';
#
# Where:
#   MASTER is the IP address or host name of the system where this script
#         is installed
#   DBUSER is the database user configured below
#   DBPWD is the database password configured below
#
# Call with -f to force purging of log files if all conditions are met
#
# Using SMTP email for Alerts
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
# An MTA daemon should not be running on a SecureSpan Gateway. To enable
# sending of notification by SMTP you must configure a smart relay host in the
# MTA configuration. Appliances running on RHEL 4 (SSG v5.0 and previous) have
# sendmail installed. Appliances running on RHEL 5 (SSG v5.1 and later) have
# exim installed. The following sections outline steps to take for each
# version.
#
# Configuring Exim
# ~~~~~~~~~~~~~~~~
#
# To configure exim to use a smarthost edit /etc/exim/exim.conf and find the
# line with "begin routers". Add the following lines immediately after "begin
# routers" line. Change "mailhost.example.com" to your smarthost:
#
# # Configure a smart host to route everything but local domain
# smarthost:
# driver = manualroute
# domains = !+local_domains
# transport = remote_smtp
# route_data = mailhost.example.com
#
# Make sure the MTA_FLAGS below is set to "-t"
#
# Configuring Sendmail
# ~~~~~~~~~~~~~~~~~~~~
#
# Configure the smarthost in sendmail by setting the DS configuration in the
# /etc/mail.sendmail.cf file and set the MTA_FLAG value below to "-Am -t"
#
# Using SNMP Trap for Alerts
# ~~~~~~~~~~~~~~~~~~~~~~~~~~
#
# To send snmp trap alerts the net-snmp-utils rpm package must be installed.
# This is not a default in the SecureSpan appliances (as of v6.2). Use scp to
# copy the rpm file to the ssgconfig user's home directory then run:
#   # rpm -Uvh ~ssgconfig/net-snmp-utils-<version info>.rpm
#
# There are two possible traps that may be sent: ERROR and INFO. The OID for
# ERROR is 1.3.6.1.4.1.17304.8.10.50 and indicates that action should be taken.
# The OID for INFO is 1.3.6.1.4.1.17304.8.10.51. No action needs to be taken
# for INFO. Layer 7 has reserved the 1.3.6.1.4.1.17304.* MIB space. To
# set the OID to something other than .50 or .51 change the values of
# OID_ERROR and OID_INFO below.
#
# Configuring crontab
# ~~~~~~~~~~~~~~~~~~~
#
# Run the command 'crontab -e' as root and add the following line (note: it
# opens the file in vi). See the crontab man page for details:
#
# to check every hour on the hour:
#    0 * * * * /usr/local/bin/manage_binlogs.sh -f
#
# to check every day at 04:15
#    15 4 * * * /usr/local/bin/manage_binlogs.sh -f
#
# Using with SNMP GET
# ~~~~~~~~~~~~~~~~~~~
#
# This script can be called using the exec extension of snmpd.conf.
#
# exec .1.3.6.1.4.1.17304.8.1 replication_monitor /usr/local/bin/manage_binlogs.sh -gf
#
# Walking that point of the MIB results in:
#
# $ snmpwalk -v 2c -n -Ou ssg62-1.l7tech.com -c l7 enterprises.17304.8.1
# SNMPv2-SMI::enterprises.17304.8.1.1.1 = INTEGER: 1
# SNMPv2-SMI::enterprises.17304.8.1.2.1 = STRING: "replication_monitor"
# SNMPv2-SMI::enterprises.17304.8.1.3.1 = STRING: "/usr/local/bin/manage_binlogs.sh -gf"
# SNMPv2-SMI::enterprises.17304.8.1.100.1 = INTEGER: 0
# SNMPv2-SMI::enterprises.17304.8.1.101.1 = STRING: "OK (purge forced)"
# SNMPv2-SMI::enterprises.17304.8.1.102.1 = INTEGER: 0
# SNMPv2-SMI::enterprises.17304.8.1.103.1 = ""
# $
#
# Obtaining database user passwords from a Layer 7 Gateway
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#
# This scripts requires two passwords, one to access the MySQL database on
# the remote system, the other to access an admin user on the local database
# for purging the binlogs if replication is working. Storing password in the
# clear in a script is not advisable. To get around this you can store
# passwords in the encrypted password store of a Layer 7 Gateway and make
# the password available via a REST service. For instructions on doing this
# please contact Layer 7 Support.
#
# Revision History: 
# ~~~~~~~~~~~~~~~~~
# Jay MacDonald - v1 - 20080123
# Jay MacDonald - v2 - 20080214
# Jay MacDonald - v3 - 20080425 - Added timestamp to messages
# Jay MacDonald - v4 - 20080529 - Added root db pwd configuration options
# Jay MacDonald - v5 - 20081111 - Fixed root db passwd bug (mising then)
# Jay MacDonald - v6 - 20091006 - Added exim option and more doc
# Jay MacDonald - v7 - 20100209 - Added snmptrap option for Notify
# Jay MacDonald - v8 - 20120522 - Added modes for SNMP walk replies and getopts
# Jay MacDonald - v9 - 20120601 - Added information for obtaining passwords from Gateway

################################################# Configurable settings

# SLAVE is either IP address or host name
SLAVE="gateway2.ca.com"			# Override with '-s <hostname>'
DBUSER="repluser"		# Override with '-u <user>'
#DBPWD="replpass"		# Override with '-p <password>'
DBPWD=`/usr/bin/wget -O- -q --no-check-certificate https://localhost:8443/getPwd?p=dbreplpwd`
ROOTDBUSER="root"		# Override with '-r <user>'
#ROOTDBPWD="7layer"		# Override with '-a <password>'
ROOTDBPWD=`/usr/bin/wget -O- -q --no-check-certificate https://localhost:8443/getPwd?p=dbadminpwd`

# NOTIFY_SMTP and NOTIFY_SNMP_TRAP methods work on error conditions only.
# If you want notifications on all runs set NOTIFY_PURGE as well
NOTIFY_SMTP_MAIL="no"		# Override with '-m yes'
NOTIFY_SNMP_TRAP="no"		# Override with '-t yes'

# NOTIFY_SNMP_GET always returns OK, FAIL or UNKNOWN with status in parenthesis
NOTIFY_SNMP_GET="no"		# Override with '-g'

# Configure email alert settings. Needed if NOTIFY_SMTP_MAIL="yes"
# Must set NOTIFY_TO. NOTIFY_CC and NOTIFY_BCC are optional.
NOTIFY_TO="test@ca.com"
NOTIFY_CC=""
NOTIFY_BCC=""

# Configure SNMP Trap alert settings. Needed if NOTIFY_SNMP_TRAP="yes"
# Must set SNMP_HOST and COMMUNITY
SNMP_HOST="SET ME"
COMMUNITY="SET ME"
# Change these if you want to use different trap OID values in 1.3.6.1.4.1.17304.8.10.
OID_ERROR="50"
OID_INFO="51"

# NOTIFY_PURGE sends notification on successful cases. It can get to be
# a bit noisy if left on...
NOTIFY_PURGE="yes"		# Override with '-q'

# Set VERBOSE to yes when debugging. Under normal operations it should
# probably be set to "no". Writes to STDERR to avoid collision with
# NOTIFY_SNMP_GET
VERBOSE="no"			# Override with '-v'

###########################################################################
################################################# End configurable settings
###########################################################################

########################################################## Define functions

clean_up() {
	if [ "$NOTIFY_SNMP_GET" == "yes" ] ; then
		echo "$SNMP_GET_MSG"
	fi

	rm -f /tmp/mb_*.$$
	exit $1
}

check_yes_no() {
	if [ "$1" == "y" -o "$1" == "Y" -o "$1" == "yes" -o "$1" == "YES" ] ; then
	   OPTARG="yes"
	   return 0
	fi
	if [ "$1" == "n" -o "$1" == "N" -o "$1" == "no" -o "$1" == "NO" ] ; then
	   OPTARG="no"
	   return 0
	fi
	echo "$0: argument for -$opt must be yes or no" >&2
	clean_up 1
}

print_help () {
	echo "$0 - Check replication status and purge binlogs"
	echo ""
	echo "Command line parameters:"
	echo "  -f		: Force purge"
	echo "  -s <hostname>	: Override SLAVE (slave host name)"
	echo "  -u <user>	: Override DBUSER (remote database replication user name)"
	echo "  -p <password>	: Override DBPWD (remote database replication user password)"
	echo "  -r <user>	: Override ROOTDBUSER (local database admin user name)"
	echo "  -a <password>	: Override ROOTDBPWD (local database admin user password)"
	echo "  -m yes|no	: Override NOTIFY_SMTP_MAIL (Notify by email)"
	echo "  -t yes|no	: Override NOTIFY_SNMP_TRAP (Notify by SNMP trap)"
	echo "  -g		: Override NOTIFY_SNMP_GET (Notify by SNMP GET) default=no"
	echo "  -q		: Override NOTIFY_PURGE (Notify when status is OK but purge was run)"
	echo "  -v		: Override VERBOSE (VERBOSE writes to STDERR)"
	echo "  -d		: Print the configured defaults and exit"
	echo "  -h		: Print this list and exit"
	echo ""
	echo "Exit status 0 if replication is working, 1 if not"
}

print_defaults () {
	echo "Default settings:"
	echo "  VERBOSE=$VERBOSE"
	echo "  FORCE=no"
	echo "  SLAVE=$SLAVE"
	echo "  DBUSER=$DBUSER"
	echo "  DBPWD=$DBPWD"
	echo "  ROOTDBUSER=$ROOTDBUSER"
	echo "  ROOTDBPWD=$ROOTDBPWD"
	echo "  NOTIFY_SMTP_MAIL=$NOTIFY_SMTP_MAIL"
	echo "  NOTIFY_SNMP_TRAP=$NOTIFY_SNMP_TRAP"
	echo "  NOTIFY_SNMP_GET=$NOTIFY_SNMP_GET"
	echo "  NOTIFY_PURGE=$NOTIFY_PURGE"
}

notify () {
	if test "$NOTIFY_SMTP_MAIL" == "yes" ; then
		echo "From: ${USER}@${HOSTNAME}" > /tmp/mb_notify.$$
		echo "To: $NOTIFY_TO" >> /tmp/mb_notify.$$
		if test -n $NOTIFY_CC ; then
			echo "Cc: $NOTIFY_CC" >> /tmp/mb_notify.$$
		fi
		if test -n $NOTIFY_BCC ; then
			echo "Bcc: $NOTIFY_BCC" >> /tmp/mb_notify.$$
		fi
		echo "Subject: $SUBJECT" >> /tmp/mb_notify.$$
		echo "" >> /tmp/mb_notify.$$
		cat /tmp/mb_message.$$ >> /tmp/mb_notify.$$

		verbose "Sending notification by email" >&2

		#cat /tmp/mb_notify.$$ | /usr/sbin/sendmail $MTA_FLAG
		/usr/bin/wget -O- -q --no-check-certificate https://localhost:8443/sendmail
	fi

	if test "$NOTIFY_SNMP_TRAP" == "yes" ; then
		verbose "Sending notification by SNMP trap" >&2

		if test -z $1 ; then
			OID=$OID_ERROR
		else
			OID=$1
		fi

		# Send the trap. Note: this is a v2c style message.
		/usr/bin/snmptrap -v 2c -c $COMMUNITY $SNMP_HOST $UPTIME 1.3.6.1.4.1.17304.8.10.${OID} \
		SNMPv2-SMI::enterprises.17304.8.10.0 s "${SUBJECT}"
	fi
}

verbose() {
	if test "$VERBOSE" == "yes" ; then
		echo $1 >&2
	fi
}

###########################################################################
################################################## End function definitions
###########################################################################

############################################ Parse the command line options

OPTS="hds:u:p:r:a:m:t:gqvf"

# First check if we need to dump the help or defaults and exit

while getopts $OPTS opt ; do
	case $opt in
	   h)	print_help
		clean_up 0
		;;

	   d)	print_defaults
		clean_up 0
		;;
	esac
done

OPTIND=1	# Reset the getopts index for the second pass

# Get the command line options
while getopts $OPTS opt ; do
	case $opt in
	   v)	VERBOSE="yes" ;;

	   f)	FORCE="yes" ;;

	   s)	SLAVE=$OPTARG ;;

	   u)	DBUSER=$OPTARG ;;

	   p)	DBPWD=$OPTARG ;;

	   r)	ROOTDBUSER=$OPTARG ;;

	   a)	ROOTDBPWD=$OPTARG ;;

	   m)	check_yes_no $OPTARG
		NOTIFY_SMTP_MAIL=$OPTARG
		;;

	   t)	check_yes_no $OPTARG
		NOTIFY_SNMP_TRAP=$OPTARG
		;;

	   g)	NOTIFY_SNMP_GET="yes" ;;

	   q)	NOTIFY_PURGE="yes" ;;

	   ?)	clean_up 1 ;;
	esac
done

############################################# Confirm minimal configuration

if test "$SLAVE" == "SET ME" ; then
	echo "Error: SLAVE is not configured"
	clean_up 1
fi

# Test for minimal settings for notifying by email
if test "$NOTIFY_SMTP_MAIL" == "yes" ; then
	# For exim set MTA_FLAG to "-t". For sendmail set it to "-Am -t"
	MTA_FLAG="-t"

	if test "$NOTIFY_TO" == "SET ME" ; then
		echo "Error: NOTIFY_TO is not configured"
		clean_up 1
	fi
fi

# Test for minimal settings for notifying by snmp trap
if test "$NOTIFY_SNMP_TRAP" == "yes" ; then
	UPTIME=`/bin/cat /proc/uptime | /bin/cut -f 1 -d '.'`

	if test "$SNMP_HOST" == "SET ME" ; then
		echo "Error: SNMP_HOST is not configured"
		clean_up 1
	fi

	if test "$COMMUNITY" == "SET ME" ; then
		echo "Error: COMMUNITY is not configured"
		clean_up 1
	fi

	if test ! -x /usr/bin/snmptrap ; then
		echo "Error: Can't find /usr/bin/snmptrap. Make sure the net-snmp-utils rpm has"
		echo "been installed."
		clean_up 1
	fi
fi

############################################################# Execute steps

# Initialize SNMP_GET_MSG
SNMP_GET_MSG="UNKNOWN"

# Get the timestamp
DATE=`date`

# Query the SLAVE STATUS from the slave database
RESULT=`mysql -h $SLAVE -u$DBUSER -p$DBPWD -e "SHOW SLAVE STATUS\G" 2>/tmp/mb_error.$$`

# Check to make sure we got the connection, notify and fail if not
if test $? -ne 0; then

	cat /tmp/mb_error.$$ | logger -t $0 --

	cat > /tmp/mb_message.$$ <<-EOM

		==> Error querying database

		Timestamp: $DATE
		Message: `cat /tmp/mb_error.$$`

		Refer to http://dev.mysql.com/doc/refman/4.1/en/error-messages-client.html
		for more information.

	EOM
	if test $VERBOSE == yes ; then
		cat /tmp/mb_message.$$ >&2
	fi

	SUBJECT="WARNING: Error querying replication status at $SLAVE"
	SNMP_GET_MSG="UNKNOWN (Error querying database. Refer to logs)"
	notify $OID_ERROR
	clean_up 1
fi

# Get the useful information from SHOW SLAVE STATUS\G and convert to variables
eval `echo "$RESULT" | \
	sed 's/^  *//' | \
	sed 's/: \(.*\)/="\1"/' | \
	grep -v '=""$' | \
	grep '='`

verbose "Master_Host = $Master_Host"
verbose "Slave_IO_Running = $Slave_IO_Running"
verbose "Slave_SQL_Running = $Slave_SQL_Running"
verbose "Master_Log_File = $Master_Log_File"
verbose ""

# Configure $ROOTDBPWD
if test -n "$ROOTDBPWD" ; then
	ROOTDBPWD=" -p$ROOTDBPWD"
fi

# Check the status of the slave and purge if OK else send notification
if test "$Slave_IO_Running" == "Yes" -a "$Slave_SQL_Running" == "Yes" ; then
	verbose "Slave at $SLAVE is functioning properly. To purge logs run:"
	verbose ""
	verbose "  mysql -u $ROOTDBUSER${ROOTDBPWD} -e \"PURGE MASTER LOGS TO '$Master_Log_File'\""
	verbose ""
	if test "$FORCE" == "yes" ; then
		verbose "Forcing purge"
		mysql -u $ROOTDBUSER${ROOTDBPWD} -e "PURGE MASTER LOGS TO '$Master_Log_File'" 2>/tmp/mb_error.$$

		if test $? -ne 0; then

			cat /tmp/mb_error.$$ | logger -t $0 --

			cat > /tmp/mb_message.$$ <<-EOM

				==> Error purging log files

				Timestamp: $DATE
				Message: `cat /tmp/mb_error.$$`

				Refer to http://dev.mysql.com/doc/refman/4.1/en/error-messages-client.html
				for more information.

			EOM
			if test "$VERBOSE" == "yes" ; then
				cat /tmp/mb_message.$$ >&2
			fi

			SUBJECT="WARNING: Error purging log files on $HOSTNAME"
			SNMP_GET_MSG="UNKNOWN (Error purging log file. Refer to logs)"
			notify $OID_ERROR
			clean_up 1
		fi

		logger -t $0 "NOTICE: Forcing purge of logs on $HOSTNAME"

		if test "$NOTIFY_PURGE" == "yes" ; then
			SUBJECT="NOTICE: Forcing purge of logs on $HOSTNAME"
			echo "Purging to $Master_Log_File" > /tmp/mb_message.$$
			echo "Timestamp: $DATE" >> /tmp/mb_message.$$
			notify $OID_INFO
		fi
		SNMP_GET_MSG="OK (purge forced)"
	else
		verbose "Not forcing purge. To force the purge call with -f"
		SNMP_GET_MSG="OK (purge not forced)"
	fi

	clean_up 0
else
	verbose "WARNING: Slave at $SLAVE is not functioning properly"
	if [ "$NOTIFY_SMTP_MAIL" == "yes" -o "$NOTIFY_SNMP_TRAP" == "yes" ] ; then
		verbose " --- This *should* have been sent by SMTP or SNMP Trap ---"
	else
		verbose " --- No one was notified! ---"
	fi

	# Create message and send

	SUBJECT="WARNING: Slave at $SLAVE is not functioning properly"

	logger -t $0 "WARNING: The database slave at $SLAVE is not functioning properly"

	cat > /tmp/mb_message.$$ <<-EOM
		WARNING: The database slave at $SLAVE is not functioning properly
		Timestamp: $DATE
	
		Here are the results of SHOW SLAVE STATUS\G:

		$RESULT
	EOM

	SNMP_GET_MSG="FAIL (Replication at $SLAVE is not functioning properly)"
	notify $OID_ERROR

	clean_up 1
fi
